;
;
;********************************
;*				*
;*	  TurboROM  v1.2	*
;*	 Download Routine	*
;*				*
;********************************
;
;
; Download - find emulator(s), load memory, set emulate mode
; c=0 if emulator(s) found and loaded
; c=1 if not found
;
download:	call	find_emulator		;find emulator on parallel port
		jnc	d_okay			;error?
		jmp	d_exit
d_okay:
		mov	al,11110100b		;put emulator(s) in load mode
		out_dx_al

		mov	bl,setup_device		;do high pre-inc if necessary
		xor	bh,bh			;to establish initial pins
		mov	cl,d_high_incs [bx]	;1,2,30,31 /32 states
		xor	ch,ch
		jcxz	d_no_high_pre
d_high_pre:	mov	al,11110110b
		out_dx_al
		mov	al,11110100b
		out_dx_al
		loop	d_high_pre
d_no_high_pre:
		cmp	setup_device,5		;do low pre-inc if 27128, 2764
		jb	d_no_low_pre		;to establish initial pin 27/28
		mov	cx,08000h		;high state
d_low_pre:	mov	al,11110000b
		out_dx_al
		mov	al,11110100b
		out_dx_al
		loop	d_low_pre
d_no_low_pre:
		mov	bl,setup_device		;get number of copies in di
		xor	bh,bh
		mov	bl,d_copies [bx]
		mov	di,bx

d_next_copy:	xor	bp,bp			;reset device address in cl:bp
		xor	cl,cl

d_next_word:	test	bp,0ffh			;new page?
		jnz	d_no_print
		test	editor_flag,1		;editor mode?
		jz	d_no_print
		mov	al,cl			;print 3-digit page number
		call	print_hex_nib
		mov	ax,bp
		mov	al,ah
		call	print_hex_byte
		sub	chr_x,3			;back up cursor for next time
d_no_print:
		mov	al,11110000b		;ce low, next address
		out_dx_al

		call	send_word		;send word to emulator(s)

		mov	bx,1111000011110100b	;load bx for quick constants

		add	bp,01h			;inc device address in cl:bp
		adc	cl,0h
		test	bp,07fffh		;if 32k roll-over, do high-inc
		jnz	d_no_high_inc

		mov	al,11110110b		;high-inc
		out_dx_al
		mov	al,bl			;inc high address, set data
		out_dx_al
		mov	al,bh			;ce high, same address
		out_dx_al
		mov	al,11110010b		;update high address outputs
		out_dx_al
		jmp	d_high_incd

d_no_high_inc:	mov	al,11110010b		;no high-inc
		out_dx_al
		mov	al,bh			;set data
		out_dx_al
		mov	al,bl
		out_dx_al
		mov	al,bh			;ce high, same address
		out_dx_al
d_high_incd:	mov	al,bl
		out_dx_al

		cmp	bp,device_size		;if not end of device, loop
		jne	d_next_word
		cmp	cl,device_size_high
		je	d_copy_done
		jmp	d_next_word

d_copy_done:	dec	di			;end of device, check if more
		jz	d_done			;copies to be made to account
		jmp	d_next_copy		;for n.c. pins (unknown states)

d_done:		mov	al,11111111b		;done, set emulate mode
		out_dx_al
		clc				;no errors

d_exit:		ret


d_high_incs	db	0h,08h,08h,0eh,0fh,0fh,0fh
d_copies	db	1,1,2,1,1,1,2
;
;
; Send word to emulator
;
send_word:	mov	si,bp			;get cl:bp into ch:si
		mov	ch,cl

		mov	bl,setup_split		;go to appropriate handler
		shl	bl,1
		xor	bh,bh
		jmp	sw_handlers [bx]

sw_handlers	dw	offset send_cache	;1x1 split
		dw	offset sw_split_1_2
		dw	offset sw_split_1_4
		dw	offset sw_split_2_1
		dw	offset sw_split_2_2
		dw	offset sw_split_4_1

sw_split_1_2:	shl	si,1			;1x2 split
		rcl	ch,1
		call	send_cache
		inc	si
		jmp	send_cache

sw_split_1_4:	shl	si,1			;1x4 split
		rcl	ch,1
		shl	si,1
		rcl	ch,1
		call	send_cache
		inc	si
		call	send_cache
		inc	si
		call	send_cache
		inc	si
		jmp	send_cache

sw_split_2_1:	call	send_cache		;2x1 split
		add	si,device_size
		adc	ch,device_size_high
		jmp	send_cache

sw_split_2_2:	shl	si,1			;2x2 split
		rcl	ch,1
		call	send_cache
		inc	si
		call	send_cache
		dec	si
		add	si,device_size
		adc	ch,device_size_high
		add	si,device_size
		adc	ch,device_size_high
		call	send_cache
		inc	si
		jmp	send_cache

sw_split_4_1:	call	send_cache		;4x1 split
		add	si,device_size
		adc	ch,device_size_high
		call	send_cache
		add	si,device_size
		adc	ch,device_size_high
		call	send_cache
		add	si,device_size
		adc	ch,device_size_high
		jmp	send_cache
;
;
; Read byte from cache at ch:si and send to emulator
;
send_cache:	push	dx			;preserve dx
		mov	bx,si			;get ch:si into dl:bx
		mov	dl,ch
		call	read_cache		;get cache byte dl:bx into al
		pop	dx

		mov	ah,al			;get al into ah

		mov	bx,0000001001111001b	;load bx for quick constants

		shr	ah,1			;bit 0
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 1
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 2
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 3
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 4
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 5
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 6
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al
		shr	ah,1			;bit 7
		mov	al,bl
		rcl	al,1
		out_dx_al
		xor	al,bh
		out_dx_al

		ret
;
;
; Find emulator on parallel port
; c=0 if found with dx holding port address, else c=1
;
find_emulator:	xor	ax,ax			;es:si points to bios list of
		mov	es,ax			;parallel port addresses
		mov	si,0408h		;starting at 0:0408h (040:08h)

		mov	bl,setup_port		;handle port according to setup
		cmp	bl,4			;if 'off', search ports
		je	fe_get_port
		dec	bl			;if 'auto', search ports
		js	fe_get_port
		shl	bl,1			;port 1, 2, or 3
		and	bx,06h			;get word offset in bx
		mov	ax,es:[si+bx]		;get port address
		mov	si,0408h+8		;force a last attempt
		jmp	fe_chk_port

fe_get_port:	lods	word ptr es:[si]	;get bios port address
fe_chk_port:	or	ax,ax			;skip test if no port
		jz	fe_bad_port
		mov	dx,ax			;port address into dx

fe_test_port:	mov	ah,069h			;send 069h

		mov	cx,8			;test for 74hc595 serial loop
fe_send:	shl	ah,1			;shift bit out
		mov	al,01111111b
		rcl	al,1			;rotate into 'out' byte
		out_dx_al
		xor	al,00000010b
		out_dx_al
		loop	fe_send

		mov	cx,8			;read byte back
fe_receive:	inc	dx
		in_al_dx			;read 'busy' line
		dec	dx
		shl	al,1
		cmc				;un-invert
		rcl	ah,1			;rotate into byte
		mov	al,11111110b
		out_dx_al
		mov	al,11111100b
		out_dx_al
		loop	fe_receive

		cmp	ah,069h			;if 069h then got port
		je	fe_exit			;emulator found, dx=port, c=0

fe_bad_port:	cmp	si,0408h+8		;try next port if not last
		jne	fe_get_port
		stc				;emulator not found, c=1

fe_exit:	push	ds			;restore es from ds
		pop	es
		ret
